﻿-- communication
local addon = PriorityList
local AceGUI = LibStub("AceGUI-3.0")

local addonName, addonTable = ...
local L = addonTable.L

local tinsert, tconcat, tremove = table.insert, table.concat, table.remove
local fmt, tostring, string_char = string.format, tostring, string.char
local select, pairs, next, type, unpack = select, pairs, next, type, unpack
local setmetatable, getmetatable, rawset, rawget = setmetatable, getmetatable, rawset, rawget
local bit_band, bit_lshift, bit_rshift = bit.band, bit.lshift, bit.rshift

-- string colors
local LIGHT_BLUE  = "|cff00ccff"
local LIGHT_RED   = "|cffff6060"
local GREEN       = "|cff71C671"
local LIGHT_GREEN = "|cff71FF71"
local YELLOW      = "|cffffff00"
local GREY        = "|cff888888"
local GOLD        = "|cffffcc00"
local RED         = "|cffff0000"
local function AddColor(str,color)
 return color..str.."|r"
end

function addon:ImportList(name,char)

 local function importchar(nm,cr)
  if not PLWish_List[cr] then PLWish_List[cr] = {} end
  PLWish_List[cr] = addon.import_var.Wish_List[nm][cr]
  addon.import_var.Wish_List[nm][cr] = nil
  if addon.import_var.Deleting_List[nm] then addon.import_var.Deleting_List[nm][cr] = nil end
 end

 if addon.import_var.Wish_List then
  if char then
   importchar(name,char)
  else
   for i,j in pairs(addon.import_var.Wish_List[name]) do
    importchar(name,i)
   end
   addon.import_var.Wish_List[name] = nil
  end
  if addon.import_var.Wish_List[name] then
   local found = false
   table.foreach(addon.import_var.Wish_List[name],function(k,v) found = true end)
   if not found then addon.import_var.Wish_List[name] = nil end --no chars left
  end
  addon:GenerateItemsList("main")
  addon:GenerateItemsList("import")
  addon:PopulateItemsList_new()
 end
end

function addon:ExcludeList(name,char)

 local function excludechar(nm,cr)
  if addon.import_var.Wish_List[nm] then addon.import_var.Wish_List[nm][cr] = nil end
  if addon.import_var.Deleting_List[nm] then addon.import_var.Deleting_List[nm][cr] = nil end
 end

 if addon.import_var.Wish_List then
  if char then
   excludechar(name,char)
  else
   addon.import_var.Wish_List[name] = nil
   addon.import_var.Deleting_List[name] = nil
  end
 end
 addon:GenerateItemsList("import")
 addon:PopulateItemsList_new()
end

-- import list tab creation
function addon:TabInit_ImportExport(container)
if container then
 Sorted_Items_List = {}

-- buttons panel
 addon.opt.upperpanel = AceGUI:Create("SimpleGroup")
 addon.opt.upperpanel:SetFullWidth(true)
 addon.opt.upperpanel:SetLayout("Flow")
 container:AddChild(addon.opt.upperpanel)
--import all button
 local btn_importall = AceGUI:Create("Button")
 btn_importall:SetText(L["impall"])
 btn_importall:SetWidth(150)
 btn_importall:SetCallback("OnClick",
  function(widget) --import all onclick
   table.foreach(addon.import_var.Wish_List,
    function(k,v)
     addon:ImportList(k)
    end)
  end)
 addon.opt.upperpanel:AddChild(btn_importall)
--end import button
 local btn_endimport = AceGUI:Create("Button")
 btn_endimport:SetText(L["clearlist"])
 btn_endimport:SetWidth(150)
 btn_endimport:SetCallback("OnClick",
  function(widget) --end import onclick
   wipe(addon.import_var.Wish_List)
   addon:PopulateItemsList_new()
  end)
 addon.opt.upperpanel:AddChild(btn_endimport)

 local btn_importreq = AceGUI:Create("Button")
 btn_importreq:SetText(L["importrequests"])
 btn_importreq:SetWidth(200)
 btn_importreq:SetCallback("OnClick",
 function(widget) --import requests onclick
  local menuFrame = CreateFrame("Frame", "PLButtonMenuFrame", UIParent, "UIDropDownMenuTemplate")
  local menu = {}
  local didsomething = false
  table.foreach(addon.import_var.requests,
   function(k,v)
    table.insert(menu,{text = L["import"].." "..L["from"]..k, notCheckable = 1, 
    func = function()
            addon:SendCommMessage("PriorityList", "::msg:accept", "WHISPER", k)
            addon.import_var.requests[k] = nil
           end})
    didsomething = true
   end)
  if didsomething then
   table.insert(menu,{text = "", isTitle = true, notCheckable = 1})
   table.insert(menu,{text = L["import"].." "..L["all"], notCheckable = 1, 
   func = function()
           table.foreach(addon.import_var.requests, 
            function(k,v)
             addon:SendCommMessage("PriorityList", "::msg:accept", "WHISPER", k)
             addon.import_var.requests[k] = nil
            end)
          end})
  end
  EasyMenu(menu, menuFrame, "cursor", 0 , 0, "MENU")
 end)
-- addon.opt.btn_endimport:SetCallback("OnEnter", function(widget) addon:SetStatus(L["endimportdesc"]) end)
-- addon.opt.btn_endimport:SetCallback("OnLeave", function(widget) addon:SetStatus("") end)
 addon.opt.upperpanel:AddChild(btn_importreq)

 addon.opt.frm_items = AceGUI:Create("ScrollFrame")
 addon.opt.frm_items:SetFullWidth(true)
 addon.opt.frm_items:SetFullHeight(true)
 addon.opt.frm_items:SetLayout("List")
 container:AddChild(addon.opt.frm_items)

 addon:GenerateItemsList("import")
 addon:PopulateItemsList_new()
end
end
-- import list tab creation end

local timeout;
function addon:exporttimer()
 timeout = timeout+1
 if timeout >= 120 then
  addon.import_var.tosend = nil
  addon:CancelTimer(addon.import_var.timer,true)
  addon.import_var.timer = nil
 end
end

function addon:StartExportTimer()
 timeout = 0;
 if addon.import_var.timer == nil then
  addon.import_var.timer = addon:ScheduleRepeatingTimer("exporttimer", 1)
 end
end


local bytetoB64 = {
    [0]="a","b","c","d","e","f","g","h",
    "i","j","k","l","m","n","o","p",
    "q","r","s","t","u","v","w","x",
    "y","z","A","B","C","D","E","F",
    "G","H","I","J","K","L","M","N",
    "O","P","Q","R","S","T","U","V",
    "W","X","Y","Z","0","1","2","3",
    "4","5","6","7","8","9","(",")"
}

local B64tobyte = {
      a =  0,  b =  1,  c =  2,  d =  3,  e =  4,  f =  5,  g =  6,  h =  7,
      i =  8,  j =  9,  k = 10,  l = 11,  m = 12,  n = 13,  o = 14,  p = 15,
      q = 16,  r = 17,  s = 18,  t = 19,  u = 20,  v = 21,  w = 22,  x = 23,
      y = 24,  z = 25,  A = 26,  B = 27,  C = 28,  D = 29,  E = 30,  F = 31,
      G = 32,  H = 33,  I = 34,  J = 35,  K = 36,  L = 37,  M = 38,  N = 39,
      O = 40,  P = 41,  Q = 42,  R = 43,  S = 44,  T = 45,  U = 46,  V = 47,
      W = 48,  X = 49,  Y = 50,  Z = 51,["0"]=52,["1"]=53,["2"]=54,["3"]=55,
    ["4"]=56,["5"]=57,["6"]=58,["7"]=59,["8"]=60,["9"]=61,["("]=62,[")"]=63
}

local encodeB64Table = {}

function PriorityList:encodeB64(str)
    local B64 = encodeB64Table
    local remainder = 0
    local remainder_length = 0
    local encoded_size = 0
    local l=#str
    for i=1,l do
        code = string.byte(str, i);
        remainder = remainder + bit_lshift(code, remainder_length);
        remainder_length = remainder_length + 8;
        while(remainder_length) >= 6 do
            encoded_size = encoded_size + 1;
            B64[encoded_size] = bytetoB64[bit_band(remainder, 63)];
            remainder = bit_rshift(remainder, 6);
            remainder_length = remainder_length - 6;
        end
    end
    if remainder_length > 0 then
        encoded_size = encoded_size + 1;
        B64[encoded_size] = bytetoB64[remainder];
    end
    return table.concat(B64, "", 1, encoded_size)
end

local decodeB64Table = {}

function PriorityList:decodeB64(str)
    local bit8 = decodeB64Table
    local decoded_size = 0
    local ch
    local i = 1
    local bitfield_len = 0
    local bitfield = 0
    local l = #str
    while true do
        if bitfield_len >= 8 then
            decoded_size = decoded_size + 1;
            bit8[decoded_size] = string_char(bit_band(bitfield, 255));
            bitfield = bit_rshift(bitfield, 8);
            bitfield_len = bitfield_len - 8;
        end
        ch = B64tobyte[str:sub(i, i)];
        bitfield = bitfield + bit_lshift(ch or 0, bitfield_len);
        bitfield_len = bitfield_len + 6;
        if i > l then
            break;
        end
        i = i + 1;
    end
    return table.concat(bit8, "", 1, decoded_size)
end

local Compresser = LibStub:GetLibrary("LibCompress")
local Encoder = Compresser:GetAddonEncodeTable()

function PriorityList:TableToString(inTable, forChat)
    local serialized = PriorityList:Serialize(inTable);
    local compressed = Compresser:Compress(serialized);
    if(forChat) then
        return PriorityList:encodeB64(compressed);
    else
        return Encoder:Encode(compressed);
    end
end

function PriorityList:StringToTable(inString, fromChat)
    local decoded
    if(fromChat) then
        decoded = PriorityList:decodeB64(inString);
    else
        decoded = Encoder:Decode(inString);
    end
    local decompressed, errorMsg = Compresser:Decompress(decoded);
    if not(decompressed) then
        return "Error decompressing: "..errorMsg;
    end
    local success, deserialized = PriorityList:Deserialize(decompressed);
    if not(success) then
        return "Error deserializing "..deserialized;
    end
    return deserialized;
end

-------------------------------------------------------
PriorityList:RegisterComm("PriorityList", function(prefix, message, distribution, sender)
 local received = PriorityList:StringToTable(PriorityList:decodeB64(message));
 if prefix == "PriorityList" and received and type(received) == "table" then -- receiving items list
 if not addon:IsSimpleMode() then -- officer mode
  if not addon.import_var.Wish_List[sender] then
   addon.import_var.Wish_List[sender] = received
  else
   table.foreach(received,
    function(k,v)
     addon.import_var.Wish_List[sender][k] = v
    end)
  end
  addon:SetStatus(L["importstatus"]..string.format(L["listreceivedS"],sender))
  addon:GenerateItemsList("import")
  if addon.opt.frm_items and addon.opt.frm_items:IsVisible() then
   if addon.opt.tab_selected == L["opt_Import/Export"] then addon:PopulateItemsList_new() end
  end
  PriorityList:SendCommMessage("PriorityList", "::msg:received", "WHISPER", sender)
 else--simple mode, importing only the playing character
--[[  smart raiders could modify others' lists.. no need to receive in simple mode
local p = UnitName("player")
  if received[p] then
   PLWish_List[p] = {}
   PLWish_List[p] = received[p]
   addon:GenerateItemsList("import")
   if addon.opt.frm_items and addon.opt.frm_items:IsVisible() then
    if addon.opt.tab_selected == L["opt_Import/Export"] then addon:PopulateItemsList_new() end
   end
  end]]--
 end-- simple mode check
 else -- receiving text message
  newmsg = message
  if prefix == "PriorityList" and strfind(newmsg, "::msg:request") then -- request sent by export btn
   if not addon:IsSimpleMode() and UnitName("player") ~= sender then -- only officers can receive lists
    local i,j = strfind(newmsg, "::msg:request")
    newmsg = string.sub(newmsg,j+1)
    if tonumber(newmsg) < PriorityList.version then
     DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>I"].." - "..string.format(L["sendrequest_Verr"],sender,AddColor(sender,GOLD).." "..L["has"]))
     PriorityList:SendCommMessage("PriorityList", "::msg:lowvers", "WHISPER", sender)
    elseif tonumber(newmsg) > PriorityList.version then
     DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>I"].." - "..string.format(L["sendrequest_Verr"],sender,AddColor(L["You"],GOLD)..L["have"]))
     PriorityList:SendCommMessage("PriorityList", "::msg:highvers", "WHISPER", sender)
    else
     addon.import_var.requests[sender] = true
     DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>I"].." - "..string.format(L["sendrequest"],sender));
    end
   end --officer mode
   elseif prefix == "PriorityList" and strfind(newmsg, "::msg:accept") then
    if addon.import_var.tosend then
     PriorityList:SendCommMessage("PriorityList", PriorityList:encodeB64(PriorityList:TableToString(addon.import_var.tosend)), "WHISPER", sender)
--     addon.import_var.tosend = nil
    end
    DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>"].." - "..L["tryingtosend"]..sender.."....");
   elseif prefix == "PriorityList" and strfind(newmsg, "::msg:received") then
    DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>"].." - "..string.format(L["listreceived"],sender));
-- version messages
   elseif prefix == "PriorityList" and strfind(newmsg, "::msg:lowvers") then
    DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>"].." - "..L["version_low"]);
   elseif prefix == "PriorityList" and strfind(newmsg, "::msg:highvers") then
    DEFAULT_CHAT_FRAME:AddMessage(L["<addon name>"].." - "..string.format(L["version_high"],AddColor(sender,GOLD)));
   end
  end
end)
-----------------
-----------------