﻿local addonName, addonTable = ...
local L = addonTable.L
local addon = PriorityList
local AceGUI = LibStub("AceGUI-3.0")

-- string colors
local LIGHT_BLUE  = "|cff00ccff"
local LIGHT_RED   = "|cffff6060"
local GREEN       = "|cff71C671"
local LIGHT_GREEN = "|cff71FF71"
local YELLOW      = "|cffffff00"
local GREY        = "|cff888888"
local GOLD        = "|cffffcc00"
local RED         = "|cffff0000"
local WHITE       = "|cffffffff"
local function AddColor(str,color)
 return color..str.."|r"
end

addon.opt = {}
local frm_main = nil
local tab_options = nil
local CHAR_ID = 999999
local IMPORT_ID = 999998
local LISTEMPTY_ID = 999997
local BOSS_SPACER = 999996
local BOSS_ID = 999995

local menuFrame = CreateFrame("Frame", "PLButtonMenuFrame", UIParent, "UIDropDownMenuTemplate")
function addon:NewButton(args,buttontype)
 local function CreateSubMenu_Link(button)

  local function mn_link_msg(channel)
  return { text = channel,notCheckable = 1, func = function()
       SendChatMessage("<<"..L["<addon name>"]..">> " , string.upper(channel))
       SendChatMessage(L["item"]..": "..button.itemLink:GetText(), string.upper(channel)) 
       SendChatMessage(L["charlisted"]..": "..args.needs, string.upper(channel)) 
       if menuFrame then menuFrame:Hide() end
      end  }
  end
 return {mn_link_msg(L["Officer"]),
         mn_link_msg(L["Guild"]),
         mn_link_msg(L["Raid"]),
         mn_link_msg(L["Party"]),
         mn_link_msg(L["Say"])}
 end

 local function CreateSubMenu_Char(name)
  local link = nil
  local n = ""

   local function mn_char_msg(channel)
    return { text = channel,notCheckable = 1, func = function()
       SendChatMessage("<<"..L["<addon name>"]..">> " , string.upper(channel))
       SendChatMessage(L["char"]..": "..name, string.upper(channel))
       local i = 0
       table.foreach(PLWish_List[name],
        function(k,v)
         i = i+1
         n,link = addon:Extr_ItemProp(v)
         if link then n = link end
         if addon:IsLooted(v) then n = n.." ("..L["Looted"]..")" end
          SendChatMessage("#"..i.." - "..n,string.upper(channel))
        end)
       if menuFrame then menuFrame:Hide() end
      end }
   end
  return {mn_char_msg(L["Officer"]),
          mn_char_msg(L["Guild"]),
          mn_char_msg(L["Raid"]),
          mn_char_msg(L["Party"]),
          mn_char_msg(L["Say"])}
 end

 local function CreateSubMenu_CharList(id)
  local result = {}
 
   --indexed foreach
   local function foreachinorder(t,f)
    local keys = {}
    for k,_ in pairs(t) do
      keys[#keys+1] = k
    end
    table.sort(keys,function (a,b)
     return string.lower(a) < string.lower(b)
    end)
    for _,k in ipairs(keys) do
     f(k,t[k])
    end
   end
   --

   local function addchar(name)
    return { text = name,notCheckable = 1, notClickable = addon:TableMatch(PLWish_List[name],id,true), func = function()
         addon:AddItem(name,id,true)
       if menuFrame then menuFrame:Hide() end
      end }
   end
   foreachinorder(PLWish_List,function(k,v)
    table.insert(result,addchar(k))
   end)
  return result
 end

 local function CreateSubMenu_Exp(name)
  local player = UnitName("player")

   local function mn_char_msg(channel)
    return { text = channel,notCheckable = 1, func = function()
     addon.import_var.tosend = {}
     addon.import_var.tosend[name] = PLWish_List[name]
     if channel ~= L["Whisper"] then
      if not addon:IsSimpleMode() then
       addon:SendCommMessage("PriorityList", "::msg:request"..PriorityList.version, channel, "")
       addon:StartExportTimer();
      else
       if player == name then -- simple mode can only send the actual playing character
        addon:SendCommMessage("PriorityList", "::msg:request"..PriorityList.version, channel, "")
        addon:StartExportTimer();
       end
      end
     else --whisper dialog
      local dialog = StaticPopup_Show("pl_textdialog")
      if dialog then
       dialog.data = "export"
       dialog.text:SetText(L["export"].." <"..name.."> "..L["to"])
      end
     end
     if menuFrame then menuFrame:Hide() end
    end}
   end

  return {mn_char_msg(L["Officer"]),
          mn_char_msg(L["Guild"]),
          mn_char_msg(L["Raid"]),
          mn_char_msg(L["Party"]),
          mn_char_msg(L["Whisper"]),}
 end

if addon.opt.frm_items then
 local button = AceGUI:Create("PriorityList_ListItem")
 button:SetTitle(args.name_col)
 button.flag:SetText(args.flag)
 wipe(button.bossdata)
 if args.itemID == CHAR_ID then --it's a character
  if args.flag ~= "msg" then
   button:SetIcon("Interface\\ICONS\\Achievement_Character_Human_Male.blp")
   button:SetBoss(args.needs_col,nil)
  else
   button:SetIcon("Interface\\ICONS\\Achievement_Boss_Illidan")
   button:SetBoss("",nil)
  end
  button:SetDescription("")
  button:SetHeight(18)
  if not addon:IsSimpleMode() then
   if args.opened then
    button.background:SetVertexColor(0.2, 0.5, 0.2, 0.25)
   else
    button.background:SetVertexColor(0.4, 0.1, 0.1, 0.25)
    button:SetTitle(args.name_col..AddColor(L["open"],LIGHT_BLUE))
   end
  end
 elseif args.itemID == IMPORT_ID then --it's a player (import interface)
  button:SetIcon("Interface\\ICONS\\INV_Letter_01.blp")
  button:SetBoss(args.needs_col,nil)
  button:SetDescription("")
  button:SetHeight(25)
  button.background:SetVertexColor(0.2, 0.8, 0.8, 0.25)
 elseif args.itemID == BOSS_ID then --it's a boss (boss list tab)
  button:SetIcon(args.icon)
  button:SetBoss("",args.bossicon)
  button:SetDescription("")
  button:SetHeight(23)
  button.background:SetVertexColor(0.4, 0.1, 0.1, 0.25)
  if args.bossdata then
   table.insert(button.bossdata,args.raid)
   table.insert(button.bossdata,args.boss)
   table.insert(button.bossdata,args.name)
  end
elseif args.itemID == BOSS_SPACER then --it's a boss (boss list tab)
  button:SetIcon("Interface\\ICONS\\INV_Sigil_Thorim.blp")
  button:SetBoss("","")
  button:SetDescription("")
  button:SetHeight(23)
  button.background:SetVertexColor(0.4, 0.4, 0.4, 0.25)
 elseif args.itemID == LISTEMPTY_ID then -- list empty
  button:SetTitle(args.name_col,GOLD)
  button:SetIcon("Interface\\ICONS\\INV_Scroll_08.blp")
  button:SetBoss("",nil)
  button:SetDescription("")
  button:SetHeight(23)
  button.background:SetVertexColor(0.5, 0.5, 0.5, 0.25)
  button:SetClick(nil)
 else--it's an item
  button:SetIcon(args.texture)
  button:SetDescription(args.needs_col)
  button:SetHeight(27)
  button.background:SetVertexColor(0.5, 0.5, 0.5, 0.25)
 if buttontype == "import" then
  if args.flag == "add" then
   button.background:SetVertexColor(0.1, 0.4, 0.1, 0.25)
   button:SetBoss(L["itemadded"],"Interface\\ICONS\\Spell_ChargePositive.blp")
   button.bossicon:SetVertexColor(0.2,1,0.2)
  elseif args.flag and args.flag:find("mod") then
   local icn = ""
   if args.flag:find("-") then
    icn = "misc_arrowdown.blp"
    button.bossicon:SetVertexColor(1,0.2,0.2)
   elseif args.flag:find("+") then
    icn = "misc_arrowlup.blp"
    button.bossicon:SetVertexColor(0.2,1,0.2)
   else
    icn = "INV_Scroll_08.blp"
    button.bossicon:SetVertexColor(1,1,1)
   end
   button:SetBoss(args.flag:sub(4),"Interface\\ICONS\\"..icn)
   button.background:SetVertexColor(0.2, 0.2, 0.4, 0.25)
  elseif args.flag == "del" then
   button.background:SetVertexColor(0.4, 0.1, 0.1, 0.25)
   button:SetBoss(L["itemremoved"],"Interface\\ICONS\\Spell_ChargeNegative.blp")
   button.bossicon:SetVertexColor(1,1,1)
  else
   button:SetBoss("","")
   button.bossicon:SetVertexColor(1,1,1)
  end--import item
 elseif buttontype == "bosslist" then
  button:SetBoss(args.sender,nil)
  button:SetHeight(20)
  button:SetDescription("")
 else
   button:SetBoss(args.bossname,args.bossicon)
   button.bossicon:SetVertexColor(1,1,1)
  end
  if args.looted then
   if buttontype ~= "import" then
    button.background:SetVertexColor(0.2, 0.2, 0.7, 0.25)
   end
   button:SetDescription(AddColor(L["Looted"],LIGHT_BLUE))
  end
 end
 button.icon:SetPoint("LEFT", button.frame, "LEFT",args.spacing,0)
 if args.link then button:SetLink(args.link) else button:SetLink("nolink") end
  button:SetClick(function()
  if GetMouseButtonClicked() == "LeftButton" then
   if buttontype == "wishlist" and args.itemID ~= CHAR_ID then
    if IsControlKeyDown() or IsAltKeyDown() then
     local real_item_id
     if args.looted then real_item_id = -args.itemID else real_item_id = args.itemID end
     local index = 0
     if PLWish_List[args.child] then
      for i,j in pairs(PLWish_List[args.child]) do
       if j == real_item_id then index = i
        break
       end
      end
     end
     if IsControlKeyDown() and index > 1 then
      addon:MoveItem(args.child,index,"up", true)
     end
     if IsAltKeyDown() and index ~= 0 and index < table.getn(PLWish_List[args.child]) then
      addon:MoveItem(args.child,index,"down", true)
     end
    end
   end  
   if IsShiftKeyDown() and ACTIVE_CHAT_EDIT_BOX then
     if args.itemID ~= CHAR_ID then
      ACTIVE_CHAT_EDIT_BOX:SetText(ACTIVE_CHAT_EDIT_BOX:GetText()..args.link)
     else
      ACTIVE_CHAT_EDIT_BOX:SetText(ACTIVE_CHAT_EDIT_BOX:GetText()..args.name)
     end
   elseif buttontype == "wishlist" then
    if args.itemID == CHAR_ID and addon.wl_folding[args.name] ~= nil then
     addon.wl_folding[args.name] = not addon.wl_folding[args.name];
     addon:PopulateItemsList_new();
    end
   elseif buttontype == "items" then
    if args.itemID == CHAR_ID and addon.items_folding[args.name] ~= nil then
     addon.items_folding[args.name] = not addon.items_folding[args.name];
     addon:PopulateItemsList_new();
    end
   elseif buttontype == "import" then
    if args.itemID == CHAR_ID and addon.import_var.folding[args.sender][args.name] ~= nil then
     addon.import_var.folding[args.sender][args.name] = not addon.import_var.folding[args.sender][args.name];
     addon:PopulateItemsList_new();
    end
   elseif buttontype == "bosslist" then
    if args.itemID == BOSS_ID then
     if args.boss then
      addon:bl_bossSet(args.raid,args.boss,args.name)
      addon:PopulateItemsList_new()
     elseif args.raid then
      addon:bl_bossSet(args.raid,args.name)
      addon:PopulateItemsList_new()
     else
      addon:bl_bossSet(args.name)
      addon:PopulateItemsList_new()
     end
    end
   end
  elseif GetMouseButtonClicked() == "RightButton" and args.flag ~= "msg" then
   local real_item_id
   if args.looted then real_item_id = -args.itemID else real_item_id = args.itemID end
  --- popup menu constructor ---
   local menu = {}
   table.insert(menu,{text = L["options"], isTitle = true, notCheckable = 1})
  -- <server query>
   if button.itemLink:GetText() == "nolink" 
  and addon.waiting_ids[args.itemID] and addon.waiting_ids[args.itemID]["notreceived"] then
   table.insert(menu,{text = L["serverquery"], notCheckable = 1, func = 
             function()
              if addon.waiting_ids[args.itemID] and addon.waiting_ids[args.itemID]["timer"] then
               addon:CancelTimer(addon.waiting_ids[args.itemID]["timer"],true)
              end
              addon.waiting_ids[args.itemID] = {["timer"] = addon:ScheduleTimer("Waiting_ID_Timer", 10, args.itemID)}
              addon.waiting_ids[args.itemID]["notreceived"] = false
              button:SetIcon("Interface\\ICONS\\Ability_Mage_TimeWarp.blp")
              button:SetTitle(L["RecItemInfo"])
             end})
   end-- server query end
  -- <move up/down> for items in wishlist
   if buttontype == "wishlist" and args.itemID ~= CHAR_ID then
    local index = 0
    if PLWish_List[args.child] then
     for i,j in pairs(PLWish_List[args.child]) do
      if j == real_item_id then index = i
       break
      end
     end
     if index > 1 then
      table.insert(menu,{text = L["move up"], notCheckable = 1, func = function() addon:MoveItem(args.child,index,"up", true) end})
     end
     if index ~= 0 and index < table.getn(PLWish_List[args.child]) then
      table.insert(menu,{text = L["move down"], notCheckable = 1, func = function() addon:MoveItem(args.child,index,"down", true) end})
     end
     table.insert(menu,{text = L["changeID"], notCheckable = 1, func = function()
                 local dialog = StaticPopup_Show("pl_textdialog")
                 if dialog then
                  dialog.data = "changeid"
                  dialog.data2 = args.child
                  dialog.data3 = real_item_id
                  dialog.text:SetText(L["changeID"].." ("..args.itemID..")")
                  dialog.editBox:SetText(args.itemID)
                 end end})
    end
   end
  -- <delete item/char> <chat link> / an import list replaces delete with import/exclude
   if args.itemID == CHAR_ID then
    if buttontype ~= "import" then
     table.insert(menu,{text = L["Add item"].." "..L["to"].." "..args.name, colorCode = LIGHT_GREEN, notCheckable = 1, func = function() addon:AddItem(args.name,0,true) end})
     table.insert(menu,{text = L["delete"].." "..L["char"], colorCode = LIGHT_RED, notCheckable = 1, func = function() addon:DeleteChar(args.name,true) end})
     table.insert(menu,{text = L["rename"].." "..L["char"], colorCode = YELLOW, notCheckable = 1, func = function()
                 local dialog = StaticPopup_Show("pl_textdialog")
                 if dialog then
                  dialog.data = "rename"
                  dialog.data2 = args.name
                  dialog.data3 = ""
                  dialog.text:SetText(L["rename"].." "..L["char"].." ("..args.name..")")
                  dialog.editBox:SetText(args.name)
                 end end})
     table.insert(menu,{text = "", isTitle = true, notCheckable = 1})
     table.insert(menu,{text = L["exportchar"], notCheckable = 1,hasArrow = true, menuList = CreateSubMenu_Exp(args.name)})
     table.insert(menu,{text = "", isTitle = true, notCheckable = 1})
     table.insert(menu,{text = L["linktochat"],notCheckable = 1, hasArrow = true,menuList = CreateSubMenu_Char(args.name)})
    else
     table.insert(menu,{text = L["import"].." "..L["char"], colorCode = LIGHT_GREEN, notCheckable = 1, func = function() addon:ImportList(args.child,args.name) end})
     table.insert(menu,{text = L["exclude"].." "..L["char"], colorCode = LIGHT_RED, notCheckable = 1, func = function() addon:ExcludeList(args.child,args.name) end})
     --no chat link for characters in import var
    end
   elseif args.itemID == IMPORT_ID then
    local suff = L["'s"]
    if strsub(args.name,strlen(args.name)) == "s" then suff = L["'"] end
     table.insert(menu,{text = L["import"].." "..args.name..suff.." "..L["wholelist"], colorCode = LIGHT_GREEN, notCheckable = 1, func = function() addon:ImportList(args.name) end})
     table.insert(menu,{text = L["exclude"].." "..args.name..suff.." "..L["wholelist"], colorCode = LIGHT_RED, notCheckable = 1, func = function() addon:ExcludeList(args.name) end})
   elseif args.itemID == LISTEMPTY_ID or args.itemID == BOSS_ID then
    -- do nothing
   else -- it's an item
    if buttontype == "import" then
     table.insert(menu,{text = L["nooptions"], notCheckable = 1})
    elseif buttontype == "items" then
     table.insert(menu,{text = L["delete"], notCheckable = 1, colorCode = LIGHT_RED, func = function() addon:DeleteItem(real_item_id, true, false, "") end})
    elseif buttontype == "bosslist" and real_item_id ~= BOSS_SPACER then
     -- "I need this!" button -- both modes
     table.insert(menu,{text = L["Ineedthis"], notClickable = addon:TableMatch(PLWish_List[UnitName("player")],real_item_id,true), notCheckable = 1, func = function() addon:AddItem(UnitName("player"),real_item_id,true) end})
     if not addon:IsSimpleMode() then
      -- "Add to char.." button -- officer mode
      table.insert(menu,{text = "", isTitle = true, notCheckable = 1})    --checked = flag droppato
      if args.link then
       table.insert(menu,{text = L["add"].." "..args.link.." "..L["to"],notCheckable = 1, hasArrow = true,menuList = CreateSubMenu_CharList(real_item_id)})
      else
       table.insert(menu,{text = L["add"].." #"..real_item_id.." "..L["to"],notCheckable = 1, hasArrow = true,menuList = CreateSubMenu_CharList(real_item_id)})
      end
     end
    elseif buttontype == "wishlist" then--tmp
     table.insert(menu,{text = L["delete"], notCheckable = 1, colorCode = LIGHT_RED, func = function() addon:DeleteItem(real_item_id, true, true, args.child) end})
     table.insert(menu,{text = "", isTitle = true, notCheckable = 1})    --checked = flag droppato
     if args.looted then
      table.insert(menu,{text = L["Looted"], checked = args.looted, func = function() addon:SetLooted(args.child,real_item_id,false,true) end})
     else
      table.insert(menu,{text = L["Looted"], checked = args.looted, func = function() addon:SetLooted(args.child,real_item_id,true,true) end})
     end
    end
    if buttontype ~= "import" and buttontype ~= "bosslist" then
     table.insert(menu,{text = "", isTitle = true, notCheckable = 1})    --checked = flag droppato
     table.insert(menu,{text = L["linktochat"],notCheckable = 1, hasArrow = true,menuList = CreateSubMenu_Link(button)})
    end
   end
   EasyMenu(menu, menuFrame, "cursor", 0 , 0, "MENU")
  end
 end) --button:SetClick
 addon.opt.frm_items:AddChild(button)
 return button
end
end

function addon:SetStatus(text)
 frm_main:SetStatusText(text)
end


local function TabInit_helpframe(container)
if container then
 addon.opt.frm_items = AceGUI:Create("ScrollFrame")
 addon.opt.frm_items:SetFullWidth(true)
 addon.opt.frm_items:SetFullHeight(true)
 addon.opt.frm_items:SetLayout("List")
 container:AddChild(addon.opt.frm_items)
 local helplabel = AceGUI:Create("Label")
 helplabel:SetFullWidth(true)
 helplabel:SetFullHeight(true)
 helplabel:SetFont("Fonts\\FRIZQT__.TTF", 14)
 helplabel:SetText(L["helptab_text"])
 addon.opt.frm_items:AddChild(helplabel)
end
end

local function TabInit_optframe(container)
if container then
 addon.opt.frm_items = AceGUI:Create("ScrollFrame")
 addon.opt.frm_items:SetFullWidth(true)
 addon.opt.frm_items:SetFullHeight(true)
 addon.opt.frm_items:SetLayout("List")
 container:AddChild(addon.opt.frm_items)

 local tmp_widget
 tmp_widget = AceGUI:Create("Label")
 tmp_widget:SetFullWidth(true)
 tmp_widget:SetFullHeight(true)
 tmp_widget:SetFont("Fonts\\FRIZQT__.TTF", 14)
 tmp_widget:SetText(L["General"])
 addon.opt.frm_items:AddChild(tmp_widget)

-- tooltip extension checkbox
 tmp_widget = AceGUI:Create("CheckBox")
 tmp_widget:SetFullWidth(true)
 tmp_widget:SetHeight(20)
 tmp_widget:SetLabel(L["opt_showttlink"])
 tmp_widget:SetType("checkbox")
 tmp_widget:SetValue(pl_options["ttlinkbtn"])
 tmp_widget:SetCallback("OnValueChanged",
  function(widget,callback,value)
   pl_options["ttlinkbtn"] = value
  end)
 addon.opt.frm_items:AddChild(tmp_widget)

-- confirm changes checkbox
 tmp_widget = AceGUI:Create("CheckBox")
 tmp_widget:SetFullWidth(true)
 tmp_widget:SetHeight(20)
 tmp_widget:SetLabel(L["opt_cnfchanges"])
 tmp_widget:SetType("checkbox")
 tmp_widget:SetValue(pl_options["confirm_edit"])
 tmp_widget:SetCallback("OnValueChanged",
  function(widget,callback,value)
   pl_options["confirm_edit"] = value
  end)
 addon.opt.frm_items:AddChild(tmp_widget)

 
-- show different difficulty items in tooltip
 tmp_widget = AceGUI:Create("CheckBox")
 tmp_widget:SetFullWidth(true)
 tmp_widget:SetHeight(20)
 tmp_widget:SetLabel(L["opt_treatdiff"])
 tmp_widget:SetType("checkbox")
 tmp_widget:SetValue(pl_options["treatdiff"])
 tmp_widget:SetCallback("OnValueChanged",
  function(widget,callback,value)
   pl_options["treatdiff"] = value
  end)
 addon.opt.frm_items:AddChild(tmp_widget)

 -- simple mode checkbox
 tmp_widget = AceGUI:Create("CheckBox")
 tmp_widget:SetFullWidth(true)
 tmp_widget:SetHeight(20)
 tmp_widget:SetLabel(L["simplemode"])
 tmp_widget:SetType("checkbox")
 tmp_widget:SetValue(pl_options["simple_mode"])
 tmp_widget:SetCallback("OnValueChanged",
  function(widget,callback,value)
   pl_options["simple_mode"] = value
   wipe(addon.import_var.Wish_List)
   addon:GenerateItemsList("main")
  end)
 addon.opt.frm_items:AddChild(tmp_widget)
 
  --spacer
  tmp_widget = AceGUI:Create("Label")
  tmp_widget:SetText(" ")
  addon.opt.frm_items:AddChild(tmp_widget)

 -- background image
 tmp_widget = AceGUI:Create("Label")
 tmp_widget:SetText(L["backgroundimg"])
 addon.opt.frm_items:AddChild(tmp_widget)
 tmp_widget = AceGUI:Create("Dropdown")
 tmp_widget:SetList(addon.Backgrounds)
 tmp_widget:SetValue(pl_options["background"])
 tmp_widget:SetCallback("OnValueChanged",
  function(widget,callback,value)
   pl_options["background"] = value
   addon:ChangeBackground(value)
--   if not Items_List then addon:GenerateItemsList("main") end
--   PriorityList:Init_OptionsFrame("main")
  end)
 addon.opt.frm_items:AddChild(tmp_widget)

--spacer
  tmp_widget = AceGUI:Create("Label")
  tmp_widget:SetText(" ")
  addon.opt.frm_items:AddChild(tmp_widget)

-- clear wish list button
 tmp_widget = AceGUI:Create("Button")
 tmp_widget:SetText(L["clearwish"])
 tmp_widget:SetWidth(150)
 tmp_widget:SetCallback("OnClick",
  function(widget) --clear wish list
   local tmp = function() 
                wipe(PLWish_List)
                wipe(Items_List)
               end
   addon:ConfirmCheck(tmp,true)
  end)
 addon.opt.frm_items:AddChild(tmp_widget)
end

end

-- collect garbage timer
local cbt
function addon:CBTimer()
 --collectgarbage()
 cbt = nil
end

local function SelectGroup(container, event, group)
 if addon.opt.edt_search then addon.opt.edt_search:SetText("") addon.opt.edt_search:Hide() addon.opt.edt_search_clear:Hide() end
 if group ==  L["itemlist"] then
  addon:ScheduleTimer("Waiting_ID_Timer", 3, "##initrefresh##")
  addon.opt.initializing = true
  container:ReleaseChildren()
  addon.opt.tab_selected = group
  addon:TabInit_ItemsList(container)
 elseif group == L["opt_bosslist"] then
  container:ReleaseChildren()
  addon.opt.tab_selected = group
  addon:TabInit_BossList(container)
 elseif group ==  L["opt_Import/Export"] then
  container:ReleaseChildren()
  addon.opt.tab_selected = group
  addon:TabInit_ImportExport(container)
 elseif group == L["opt_wish"] then
  container:ReleaseChildren()
  addon.opt.tab_selected = group
  addon:TabInit_WishList(container)
 elseif group == L["options"] then
  container:ReleaseChildren()
  addon.opt.tab_selected = group
  TabInit_optframe(container)
 elseif group == L["help"] then
  container:ReleaseChildren()
  addon.opt.tab_selected = group
  TabInit_helpframe(container)
 end
 -- just to avoid collectgarbage() spam
 if not cbt then cbt = addon:ScheduleTimer("CBTimer", 5) end
end


local background
 -- frame closed by pressing the escape key
local old_CloseSpecialWindows
-- main options frame creation
local function Init_MainFrame()

 if Wish_List then addon:GenerateItemsList("main") end
 if addon.import_var.Wish_List then addon:GenerateItemsList("import") end

 frm_main = AceGUI:Create("Frame")
 frm_main.frame:SetBackdropColor(0,0,0,1)
 frm_main.frame:SetFrameStrata("HIGH")

 frm_main:SetTitle(L["<addon name>"])
 frm_main.frame.texture = frm_main.frame:CreateTexture()
 frm_main.frame.texture:SetPoint("TOPLEFT", frm_main.frame ,"TOPLEFT", 10, -10)
 frm_main.frame.texture:SetPoint("BOTTOMRIGHT", frm_main.frame ,"BOTTOMRIGHT", -10, 10)
 frm_main.frame.texture:SetTexture(0,0,0,1)
 addon:SetStatus("",frm_main)

 -- frame closed by pressing the escape key
  if not old_CloseSpecialWindows then
   old_CloseSpecialWindows = CloseSpecialWindows
   CloseSpecialWindows = function()
    local found = old_CloseSpecialWindows()
    if frm_main.frame then
     frm_main.frame:Hide()
     return true
    end
   return found
  end
 end
 frm_main:SetLayout("Flow")
 frm_main:SetCallback("OnClose",
 function(widget)
  addon:CloseMainOptionsFrame()
 end)

-- current raid combo
 local tmp_widget = AceGUI:Create("Label")
 tmp_widget:SetText(L["current_raid"])
 tmp_widget:SetFullWidth(true)
 frm_main:AddChild(tmp_widget)
 tmp_widget = AceGUI:Create("Dropdown")
 local bosslist = {}
 table.foreach(addon.boss_drops_table,function(k,v)
   bosslist[k] = k
  end)
 bosslist[L["all_raids"]] = L["all_raids"]
 tmp_widget:SetList(bosslist)
 tmp_widget:SetValue(pl_options["current_raid"])
 tmp_widget:SetCallback("OnValueChanged",
  function(widget,callback,value)
   pl_options["current_raid"] = value
   addon:PopulateItemsList_new()
  end)
 frm_main:AddChild(tmp_widget)
-- current raid combo end

 addon.opt.frm_upperbuttons = AceGUI:Create("InlineGroup")
 addon.opt.frm_upperbuttons:SetFullWidth(true)
 addon.opt.frm_upperbuttons:SetHeight(60)
 frm_main:AddChild(addon.opt.frm_upperbuttons)

 if not addon.opt.edt_search then
  addon.opt.edt_search = CreateFrame("editbox", "PriorityListSearchInput", addon.opt.frm_upperbuttons.frame, "InputBoxTemplate")
  local edt_search = addon.opt.edt_search
  edt_search:SetAutoFocus(false)
  edt_search:SetText("")
  edt_search:SetScript("OnTextChanged",   function(...) addon:PopulateItemsList_new() end)
  edt_search:SetScript("OnEnterPressed",  function(...) edt_search:ClearFocus() end)
  edt_search:SetScript("OnEscapePressed", function(...) edt_search:SetText("") edt_search:ClearFocus() end)
  edt_search:SetScript("OnEditFocusLost", function(...) collectgarbage() end) --generating some garbage with every refresh
  edt_search:SetWidth(150)
  edt_search:SetPoint("BOTTOMLEFT", addon.opt.frm_upperbuttons.frame, "TOPLEFT", 2, -36)
  edt_search:SetPoint("TOPLEFT", addon.opt.frm_upperbuttons.frame, "TOPLEFT", 2, -18)
  edt_search:SetTextInsets(16, 0, 0, 0)
  local searchIcon = edt_search:CreateTexture(nil, "overlay")
  searchIcon:SetTexture("Interface\\Common\\UI-Searchbox-Icon")
  searchIcon:SetVertexColor(0.6, 0.6, 0.6)
  searchIcon:SetWidth(14)
  searchIcon:SetHeight(14)
  searchIcon:SetPoint("left", edt_search, "left", 3, -1)
  edt_search:SetFont("Fonts\\FRIZQT__.TTF", 10)
  addon.opt.edt_search_clear = CreateFrame("BUTTON", nil, addon.opt.frm_upperbuttons.frame)
  local edt_search_clear  = addon.opt.edt_search_clear
  edt_search_clear:SetWidth(12)
  edt_search_clear:SetHeight(12)
  edt_search_clear:SetPoint("left", edt_search, "right", 0, -1)
  edt_search_clear:SetNormalTexture("Interface\\Common\\VoiceChat-Muted")
  edt_search_clear:SetScript("OnClick", function() edt_search:SetText("") edt_search:ClearFocus() end)
 end -- edt_search creation 
 
                         -- options tabs
 local opt_chartab = {{text = L["opt_wish"], value = L["opt_wish"] },
                      {text = L["itemlist"], value = L["itemlist"]},
                      {text = L["opt_bosslist"], value = L["opt_bosslist"]},
                      {text = L["opt_Import/Export"], value = L["opt_Import/Export"] },
                      {text = L["options"], value = L["options"] },
                      {text = L["help"], value = L["help"] }}
 tab_options = AceGUI:Create("TabGroup")
 tab_options:SetLayout("Flow")
 tab_options:SetTabs(opt_chartab)
 tab_options:SetCallback("OnGroupSelected", SelectGroup)
 if table.getn(opt_chartab) > 0 then tab_options:SelectTab(opt_chartab[1].text) end
 tab_options:SetFullWidth(true)
 tab_options:SetFullHeight(true)
 background = tab_options.frame:CreateTexture(nil, "BACKGROUND")
 tab_options.frame.background = background

 background:SetTexture(pl_options["background"])

 background:SetBlendMode("DISABLE")
 background:SetVertexColor(1, 1, 1, 0.25)
 background:SetAllPoints(tab_options.frame)
 frm_main:AddChild(tab_options)
 
 frm_main:Show()

-- do not refresh the list for every item at start
 addon:ScheduleTimer("Waiting_ID_Timer", 2, "##initrefresh##")
 addon.opt.initializing = true
end -- main frame creation end

function addon:ChangeBackground(new)
 background:SetTexture(new)
end

function addon:CloseMainOptionsFrame()
 if frm_main.frame.texture then frm_main.frame.texture:SetTexture(nil) end
 if tab_options then tab_options.frame.background:SetTexture(nil) end
 if addon.opt.edt_search then addon.opt.edt_search:Hide() addon.opt.edt_search_clear:Hide() end
 if addon.opt.frm_items then addon.opt.frm_items = nil end
 if menuFrame then menuFrame:Hide() end
 AceGUI:Release(frm_main)
 collectgarbage()
 CloseSpecialWindows = old_CloseSpecialWindows
 old_CloseSpecialWindows = nil
end

function addon:Init_OptionsFrame(arg)
 if arg == "main" then
  if frm_main then
   if not frm_main:IsVisible() then
    Init_MainFrame()
   end
  else
   Init_MainFrame()
  end
 end
end